"""
querysets is a dictionary you can find in application urls.py
with the related default model queryset.
"""

from django import forms as forms_django
from django.conf import settings
from django.views.generic import create_update
import django.views.generic.list_detail as dj_list_detail
#import vov.views.create_update as create_update
from vov.functions import tools
from vov.templatetags import fast_search
from vov.template import mako_loader 
from django.db.models import get_model
from django.contrib.auth.views import redirect_to_login
from django.template import RequestContext
from django.http import Http404, HttpResponse, HttpResponseRedirect
from django.db.models import FileField, BooleanField
from django.utils.translation import ugettext
from django.contrib import messages
from django.views.generic.create_update import apply_extra_context

try:
    LOCATION = settings.LOCATION
except:
    LOCATION = '/'

def get_app_label(request):
    path = request.META['PATH_INFO']
    path = path.replace(settings.LOCATION, '', 1)
    url_pieces = path.split('/')
    app_label  = url_pieces[0]

    return app_label

def create_object(request, model, **kws):
    """a wrapper to create_object that uses mako and adds path"""

    if 'app_label' in kws:
        app_label = kws['app_label']
        del kws['app_label']
    else:
        app_label = get_app_label(request)

    model_instance = get_model(app_label, model)
    app = app_label

    # if add is a popup it uses the fake detail template to close it
    if 'popup' in getattr(request, request.method):
        post_save_redirect = '/views/close_popup'
    else:
        post_save_redirect = '%s%s/%s/%%(%s)s/' % (
            LOCATION, app, model, model_instance._meta.pk.name) 

    kw = {
          'template_name' : '%s/%s_form.mako' % (app,
                                         model_instance._meta.object_name.lower()),
          'template_loader' : mako_loader, 
          'extra_context'   : {},
          'post_save_redirect' : post_save_redirect,
          }
    kw.update(kws)

    # extra_context is a dict so can be alreadypopulated in kws
    kw['extra_context']['path'] = request.path

    return create_update.create_object(request, model_instance, **kw) 

def update_object(request, model, object_id, ext='_update', **kws):
    """a wrapper to update_object that uses mako and adds path"""

    app_label = get_app_label(request)
    model_instance = get_model(app_label, model)
    app = app_label
    
    tmpl_name_update = '%s/%s_form%s.mako' % (
        app, model_instance._meta.object_name.lower(), ext)
    tmpl_name = '%s/%s_form.mako' % (
        app, model_instance._meta.object_name.lower())

    kw = {
          'template_name' : (tmpl_name_update, tmpl_name),
          'template_loader' : mako_loader,
          'template_object_name' : 'obj',  
          'extra_context'   : {},
          'post_save_redirect' : '%s%s/%s/%s/' % (
                                         LOCATION, app, model, object_id),
          'object_id' : object_id,
          }
    kw.update(kws)

    if 'popup' in getattr(request, request.method):
            kw['post_save_redirect'] = '/views/close_popup'
    
    return update_object_sd(request, model_instance, **kw)


def update_object_sd(request, model=None, object_id=None, slug=None,
        slug_field='slug', template_name=None, template_loader=mako_loader,
        extra_context=None, post_save_redirect=None, login_required=False,
        context_processors=None, template_object_name='object',
        form_class=None):
    """
    Generic object-update function.

    Templates: ``<app_label>/<model_name>_form.html``
    Context:
        form
            the form for the object
        object
            the original object being edited
    """
    if extra_context is None: extra_context = {}
    if login_required and not request.user.is_authenticated():
        return redirect_to_login(request.path)

    model, form_class = create_update.get_model_and_form_class(model, form_class)
    obj = create_update.lookup_object(model, object_id, slug, slug_field)

    if request.method == 'POST':
        ## the next 2 line are the only difference from official 
        INCREMENTED_POST = add_missing_values_from_obj(request, obj)
        form = form_class(INCREMENTED_POST, request.FILES, instance=obj)

        if form.is_valid():
            obj = form.save()
            msg = ugettext("The %(verbose_name)s was updated successfully.") %\
                                    {"verbose_name": model._meta.verbose_name}
            messages.success(request, msg, fail_silently=True)
            return create_update.redirect(post_save_redirect, obj)
    else:
        form = form_class(instance=obj)

    if not template_name:
        template_name = "%s/%s_form.html" % (model._meta.app_label, model._meta.object_name.lower())
    t = template_loader.get_template(template_name)
    c = RequestContext(request, {
        'form': form,
        template_object_name: obj,
    }, context_processors)
    create_update.apply_extra_context(extra_context, c)
    response = HttpResponse(t.render(c))
    create_update.populate_xheaders(request, response, model, getattr(obj, obj._meta.pk.attname))
    return response

def add_missing_values_from_obj(request, obj):
    """
    Return a MutiValueDict equivalent to request.POST but with values taken from
    obj if missing from POST
    """

    from django.utils.datastructures import MultiValueDict
    data = MultiValueDict()
    boolean_fields = []
        
    for f in obj._meta.fields:
        data.setlist(f.name, [f.value_from_object(obj)])
        if isinstance(f, BooleanField):
            boolean_fields.append(f.name)
    
    for f in obj._meta.many_to_many:
        data.setlist(f.attname,  [getattr(x, x._meta.pk.name) for x in f.value_from_object(obj)])

    ## IMPORTANT: we don't want to overwrite values that are not from POST
    for i in request.POST:
        data.setlist(i, request.POST.getlist(i))

    for i in boolean_fields:
        if i not in request.POST:
            data.setlist(i, [False])

    return data
    
def delete_object(request, model, object_id, **kws):
    """a wrapper to delete_object that uses mako and adds path
    adds two flags: reload and next_url
    """
    next_url = request.GET.get('next_url', None)
    post_delete_redirect = '/views/close_popup/'

    ## next_url my be '' to trigger default name
    if next_url is not None:
        if next_url:
            post_delete_redirect += next_url
        else:
            app_label = get_app_label(request)
            model_instance = get_model(app_label, model)
            app = app_label

            post_delete_redirect += '?next_url=%s%s/%s/' % (LOCATION, app, model)


    kw = {
          'template_name' :  'delete_confirmation_popup.mako',
          'template_loader' : mako_loader,
          'template_object_name' : 'obj',  
          'extra_context' : {'path' : request.path},
          'post_delete_redirect' : post_delete_redirect,
          'object_id' : object_id,
          }
    kw.update(kws)

    return create_update.delete_object(request, model_instance, **kw)
#    def delete_object(request, model, post_delete_redirect,
#        object_id=None, slug=None, slug_field=None, template_name=None,
#        template_loader=loader, extra_context=None,
#        login_required=False, context_processors=None, template_object_name='object'):

class Obj(object):
    """A wrapper obj for a deletion of objects non stord in db"""
    
    confirmation_template_name = 'static/delete_confirmation.mako'
    post_delete_redirect = '/views/close_popup/'
    message = ''
    value = ''
    extra_form = ''
    
    def delete(self, request):
        raise NonImplementedError('You Should implement a delete method')

    def __str__(self):
        return unicode(self.value)

    def __repr__(self):
        return u"<%s: %s>" % (self.__class__.__name__, self)

def delete_non_model(request, obj, post_delete_redirect=None, 
        template_name='static/delete_confirmation.mako', 
        template_loader=mako_loader, extra_context=None, login_required=False,
        context_processors=None, template_object_name='obj'):
    """
    Generic object-delete function.

    The given template will be used to confirm deletetion if this view is
    fetched using GET; for safty, deletion will only be performed if this
    view is POSTed.

    Templates: ``<app_label>/<model_name>_confirm_delete.html``
    Context:
        object
            the original object being deleted

    :param obj: the 'obj' to be delete with attributes:
       * delete() to delete the object
       * message (possibly created by delete())
       * __str__ for template rendering
       * (possibly) confirmation_template_name: a template name to be used for 
       * (possibly) confirmation_message: a message for confirmation rendering template
    :param msg: the message to be queued to the user
    :param template_name: the template name for the confirmation
    """
    assert isinstance(obj, Obj)
    if extra_context is None: extra_context = {}
    if login_required and not request.user.is_authenticated():
        return redirect_to_login(request.path)

#    obj = lookup_object(model, object_id, slug, slug_field)

    if request.method == 'POST':
        obj.delete()
#        msg = ugettext("The %(verbose_name)s was deleted.") %\
#                                    {"verbose_name": "host " + ip}
#                                    {"verbose_name": model._meta.verbose_name}
        messages.success(request, obj.message, fail_silently=True)
        return HttpResponseRedirect(post_delete_redirect or obj.post_delete_redirect)
    else:
        if not template_name:
#            template_name = "%s/%s_confirm_delete.html" % (model._meta.app_label, model._meta.object_name.lower())
            obj.confirmation_template_name
            
        t = template_loader.get_template(template_name)
        c = RequestContext(request, {
            template_object_name: obj,
        }, context_processors)
        apply_extra_context(extra_context, c)
        response = HttpResponse(t.render(c))
#        populate_xheaders(request, response, model, getattr(obj, obj._meta.pk.attname))
        return response



def object_list(request, queryset, **kw):
    """a wrapper to object_list that uses mako and adds path in the context"""

    if 'login_required' in kw:
        if not request.user.is_authenticated():
            return redirect_to_login(request.path)
        else:
            del kw['login_required']
    if 'extra_context' in kw:
        kw['extra_context']['path'] = request.path
    else:
        kw['extra_context'] = {'path' : request.path}

    new_kw = {
              'paginate_by' : 30,
              'template_object_name' : 'obj',  
              'template_name' : '%s/%s_list.mako' %
          (queryset.model._meta.app_label, queryset.model._meta.object_name.lower()),
              'template_loader' : mako_loader,
             }
    
    new_kw.update(kw)
    kw = new_kw

    if 'app_label' in kw:
        del kw['app_label']

    return dj_list_detail.object_list(request, queryset, **kw)


#    def object_list(request, queryset, paginate_by=None, page=None,
#        allow_empty=False, template_name=None, template_loader=loader,
#        extra_context=None, context_processors=None, template_object_name='object',
#        mimetype=None):

def jobject_list(request, queryset, model, form, filter_cnd, order_by, **kw):
    """wrapper of object_list that adds filtering and order_by
    NOTE: view is no longer used!
    """
    #import pdb; pdb.set_trace()
    from vov.views.fast_search import search2
    filter_cnd, order_by, form =  search2(request, model, order_by=order_by, extra_filter=filter_cnd, **kw)

    if filter_cnd:
        queryset = queryset.filter(**filter_cnd)

    if order_by:
        queryset = queryset.order_by(order_by)

    if 'extra_context' not in kw:
        kw['extra_context'] = {}

    kw['extra_context'].update({
        'form'     : form,
        'order_by' : order_by,
        })

    return object_list(request, queryset, **kw)

def jobject_list_js(request, model, **kw):
    """ return a template for the list view that use ext Grid """
    from vov.views.fast_search import search2
    filter_cnd, order_by, form =  search2(request, model, **kw)
    
    app_label = get_app_label(request)
    model_instance = get_model(app_label, model)
    template_name = '%s/%s_list.mako' % (model_instance._meta.app_label, 
                                         model_instance._meta.object_name.lower())

    if 'app_label' in kw:
        del kw['app_label']

    if 'template_name' in kw:
        template_name = '%s/%s.mako' % (model_instance._meta.app_label, kw['template_name'].lower())
        del kw['template_name']


    c = RequestContext(request, **kw)
    c['form'] = form
            
    t = mako_loader.get_template(template_name)
    return HttpResponse(t.render(c))

# FIX ME --> cambiare tt le applicazioni che chiamano jnewobject_list
#            e chiamare jobject_list_js
#
jnewobject_list = jobject_list_js

def object_detail(request, model, object_id, queryset={}, ext='', **kws ):
    """a wrapper to object_detail that uses mako and adds path"""

    #import pdb; pdb.set_trace()
    
    if 'login_required' in kws:
        if kws['login_required'] == True:
            if not request.user.is_authenticated():
                return redirect_to_login(request.path)
        kws.pop('login_required')

    app_label = get_app_label(request)
    model_instance = get_model(app_label, model)
    app = app_label

    if model in queryset:
        q = model_instance.objects.filter(
                     **fast_search.condition2dictionary( querysets[model] ))
    else:
        q = model_instance.objects.all()

    template_name = '%s/%s_detail%s.mako' % (
        app, model_instance._meta.object_name.lower(), ext)

    kw = {
          'queryset' : q,
#          'template_name' : (template_name, 'default_detail.mako') ,
          'template_name' : template_name, 
          'template_loader' : mako_loader,
          'template_object_name' : 'obj',  
          'extra_context' : {},
          'object_id' : object_id,
          }
    kw.update(kws)

    return dj_list_detail.object_detail(request, **kw)    

def render_template(request, template_name, **kw):
    c = RequestContext(request)

    for key, value in kw.items():
        c[key] = value
        
    t = mako_loader.get_template(template_name)
    return HttpResponse(t.render(c))

# def create_object_popup(request, model=None, **kw):

#     model_instance = tools.dyn_import( model )
#     app = model_instance._meta.app_label

#     kws = {'template_name' : '%s/%s_form_popup.mako' % (app,
#     		model_instance._meta.object_name.lower()),
# 	   'post_save_redirect' : '%s%s/%s/reload/' % (	LOCATION,
# 	   		app,
# 			model_instance._meta.object_name.lower()),}

#     kw.update(kws)

#     return create_object(request, model, **kw)

def base_url(request):
    if request.is_secure():
        secure = 's'
    else:
        secure = ''
        
    return "http%s://%s%s" % (
        secure, request.META['HTTP_HOST'], settings.LOCATION)


def close_popup(request, **kw):
    from vov.template.mako_loader import render_to_response
    template_name = '/close_popup.mako'
    next_url = request.GET.get('next_url', None)
    no_reload = request.GET.get('no_reload', None)
    if next_url:
        next_url = "%s%s" % (base_url(request), next_url.lstrip('/'))
    return render_to_response(
        template_name, {'request' : request, 'no_reload' : no_reload,
                        'next_url': next_url})
